%define	auxarches i586 i686
%define enablekernel 2.4.0
%define enablekernel2 2.4.1

Summary:	The GNU libc libraries.
Name:		glibc
Version:	2.2.5
Release:	1.3.0
Copyright:	LGPL
Group:		System Environment/Libraries
Source0:	%{name}-%{version}-rh-42.tar.bz2
#Source0:	%{name}-%{version}.tar.gz
#Source1:	%{name}-linuxthreads-%{version}.tar.gz
Buildroot:	%{_tmppath}/glibc-%{release}-root
Obsoletes:	zoneinfo, libc-static, libc-devel, libc-profile, libc-headers,
Obsoletes:	linuxthreads, gencat, locale, ldconfig, locale-ja
Provides:	ldconfig
Autoreq:	false
Requires:	glibc-common = %{version}-%{release}
Prereq:		basesystem
Conflicts:	kernel < 2.4.0
Patch0:		glibc-kernel-2.4.patch
Patch1:		glibc-2.2.5.patch
Patch2:		glibc-2.2.5-security.patch
Patch3:		glibc-2.2.5-getdents.patch
Patch4:		glibc-2.2.5-xdr_array.patch
Patch5:		glibc-2.2.5-calloc.patch
Patch6:		glibc-2.2.5-dl-environ.patch
Patch7:		glibc-2.2.5-wprintf.patch
Patch8:		glibc-2.2.5-maxpacket.patch
Patch9:		glibc-2.2.5-setrlimit.patch

Patch10:	glibc-2.2.2-openbsd-freebsd-owl-fts.diff
Patch11:	glibc-2.2.3-nsswitch-gd.patch
Patch12:	glibc-2.2.5-more-xdr-fixes-gd-1.diff

%description
The glibc package contains standard libraries which are used by
multiple programs on the system. In order to save disk space and
memory, as well as to make upgrading easier, common system code is
kept in one place and shared between programs. This particular package
contains the most important sets of shared libraries: the standard C
library and the standard math library. Without these two libraries, a
Linux system will not function.

%package devel
Summary:	Header and object files for development using standard C libs.
Group:		Development/Libraries
Conflicts:	texinfo < 3.11
Prereq:		/sbin/install-info
Obsoletes:	libc-debug, libc-headers, libc-devel, linuxthreads-devel
Obsoletes:	glibc-debug
Prereq:		kernel-headers
Requires:	kernel-headers >= 2.2.1, %{name} = %{version}
Autoreq:	true

%description devel
The glibc-devel package contains the header and object files necessary
for developing programs which use the standard C libraries (which are
used by nearly all programs).  If you are developing programs which
will use the standard C libraries, your system needs to have these
standard header and object files available in order to create the
executables.

%package profile
Summary:	The GNU libc libraries, including support for gprof profiling.
Group:		Development/Libraries
Obsoletes:	libc-profile
Autoreq:	true

%description profile
The glibc-profile package includes the GNU libc libraries and support
for profiling using the gprof program.  Profiling is analyzing a
program's functions to see how much CPU time they use and determining
which functions are calling other functions during execution.  To use
gprof to profile a program, your program needs to use the GNU libc
libraries included in glibc-profile (instead of the standard GNU libc
libraries included in the glibc package).

%package common
Summary:	Common binaries and locale data for glibc
Conflicts:	%{name} < %{version}
Conflicts:	%{name} > %{version} 
Autoreq:	false
Group:		System Environment/Base

%description common
The glibc-common package includes common binaries for the GNU libc
libraries, as well as national language (locale) support and timezone
databases.


%prep
%setup -q
%ifarch ia64 sparc64 s390x %{auxarches}
# If we are building enablekernel 2.4.1 glibc on older kernel,
# we have to make sure no binaries compiled against that glibc
# are ever run
case `uname -r` in
[01].*|2.[0-3]*|2.4.0*)
%patch0 -p1
;; esac
%endif

%patch1 -p1
%patch2 -p1
%patch3 -p1
%patch4 -p1
%patch5 -p1
%patch6 -p1
%patch7 -p1
%patch8 -p1
%patch9 -p1
#%patch10 -p1
%patch11 -p1
%patch12 -p1

perl -pi -e 'm/PACKET.*1024/ and s/1024/65536/' \
  `find resolv glibc-compat -name \*.c`

# Remove .orig files.
# find . -type f -size 0 -o -name "*.orig" -exec rm -f {} \;


%build
  rm -rf build-%{_target_cpu}-linux
  mkdir build-%{_target_cpu}-linux ; cd build-%{_target_cpu}-linux

  GCC=gcc
  BuildFlags="$RPM_OPT_FLAGS -D__USE_STRING_INLINES -fstrict-aliasing"
  BuildFlags="$BuildFlags -freorder-blocks -DNDEBUG=1"
  EnableKernel="--enable-kernel=%{enablekernel}"

# Disable profiling for archs other then i386.
%ifarch %{auxarches}
  EnableKernel="$EnableKernel --disable-profile"
%endif

  echo "$BuildFlags" > ../BuildFlags
  CC="$GCC" CFLAGS="$BuildFlags -g -O3" ../configure --prefix=%{_prefix} \
	--enable-add-ons=yes --without-cvs $EnableKernel \
	%{_target_cpu}-engarde-linux

  if [ -x /usr/bin/getconf ] ; then
    numprocs=$(/usr/bin/getconf _NPROCESSORS_ONLN)
    if [ $numprocs -eq 0 ]; then
      numprocs=1
    fi
  else
    numprocs=1
  fi

  make -j$numprocs -r CFLAGS="$BuildFlags -g -O3" PARALLELMFLAGS=-s


%install
  rm -rf $RPM_BUILD_ROOT
  mkdir -p $RPM_BUILD_ROOT
  make install_root=$RPM_BUILD_ROOT install -C build-%{_target_cpu}-linux

  cd build-%{_target_cpu}-linux && \
	make install_root=$RPM_BUILD_ROOT install-locales -C ../localedata objdir=`pwd` && \
	cd ..

#%ifarch i686
#rm -rf build-%{_target_cpu}-linux2.4
#mkdir build-%{_target_cpu}-linux2.4 ; cd build-%{_target_cpu}-linux2.4
#GCC=gcc
#BuildFlags=`cat ../BuildFlags`
#EnableKernel="--enable-kernel=%{enablekernel2} --disable-profile"
#CC="$GCC" CFLAGS="$BuildFlags -g -O3" ../configure --prefix=%{_prefix} \
#	--enable-add-ons=yes --without-cvs $EnableKernel \
#	%{_target_cpu}-engarde-linux
#
#
#if [ -x /usr/bin/getconf ] ; then
#  numprocs=$(/usr/bin/getconf _NPROCESSORS_ONLN)
#  if [ $numprocs -eq 0 ]; then
#    numprocs=1
#  fi
#else
#  numprocs=1
#fi
#make -j$numprocs -r CFLAGS="$BuildFlags -g -O3" PARALLELMFLAGS=-s
#mkdir -p $RPM_BUILD_ROOT/lib/%{_target_cpu}/
#cp -a libc.so $RPM_BUILD_ROOT/lib/%{_target_cpu}/`basename $RPM_BUILD_ROOT/lib/libc-*.so`
#ln -sf `basename $RPM_BUILD_ROOT/lib/libc-*.so` $RPM_BUILD_ROOT/lib/%{_target_cpu}/`basename $RPM_BUILD_ROOT/lib/libc.so.*`
#cp -a math/libm.so $RPM_BUILD_ROOT/lib/%{_target_cpu}/`basename $RPM_BUILD_ROOT/lib/libm-*.so`
#ln -sf `basename $RPM_BUILD_ROOT/lib/libm-*.so` $RPM_BUILD_ROOT/lib/%{_target_cpu}/`basename $RPM_BUILD_ROOT/lib/libm.so.*`
#cp -a linuxthreads/libpthread.so $RPM_BUILD_ROOT/lib/%{_target_cpu}/`basename $RPM_BUILD_ROOT/lib/libpthread-*.so`
#ln -sf `basename $RPM_BUILD_ROOT/lib/libpthread-*.so` $RPM_BUILD_ROOT/lib/%{_target_cpu}/`basename $RPM_BUILD_ROOT/lib/libpthread.so.*`
#strip -R .comment $RPM_BUILD_ROOT/lib/{libc,libm,libpthread}-*.so
#cd ..
#%endif

  # compatibility hack: this locale has vanished from glibc, but some other
  # programs are still using it. Normally we would handle it in the %pre
  # section but with glibc that is simply not an option
  mkdir -p $RPM_BUILD_ROOT%{_prefix}/lib/locale/ru_RU/LC_MESSAGES

  # Remove the files we don't want to distribute
  rm -f $RPM_BUILD_ROOT%{_prefix}/%{_lib}/libNoVersion*

  # the man pages for the linuxthreads require special attention
  make -C linuxthreads/man
  mkdir -p $RPM_BUILD_ROOT%{_mandir}/man3
  install -m 0644 linuxthreads/man/*.3thr $RPM_BUILD_ROOT%{_mandir}/man3
  gzip -9nvf $RPM_BUILD_ROOT%{_mandir}/man3/*

  if [ -d $RPM_BUILD_ROOT%{_prefix}/info -a "%{_infodir}" != "%{_prefix}/info" ]; then
    mkdir -p $RPM_BUILD_ROOT%{_infodir}
    mv -f $RPM_BUILD_ROOT%{_prefix}/info/* $RPM_BUILD_ROOT%{_infodir}
    rm -rf $RPM_BUILD_ROOT%{_prefix}/info
  fi

  gzip -9nvf $RPM_BUILD_ROOT%{_infodir}/libc*

  ln -sf libbsd-compat.a $RPM_BUILD_ROOT%{_prefix}/%{_lib}/libbsd.a

  install -m 644 nss/nsswitch.conf $RPM_BUILD_ROOT/etc/nsswitch.conf

  # Take care of setuids
  # -- new security review sez that this shouldn't be needed anymore
  #chmod 755 $RPM_BUILD_ROOT%{_prefix}/libexec/pt_chown

  # Don't include ld.so.cache
  rm -f $RPM_BUILD_ROOT/etc/ld.so.cache

  # Include ld.so.conf
  >$RPM_BUILD_ROOT/etc/ld.so.conf
  chmod 644 $RPM_BUILD_ROOT/etc/ld.so.conf

# Install the upgrade program
# XXX

  # Strip binaries
  strip -R .comment $RPM_BUILD_ROOT/sbin/* || :
  strip -R .comment $RPM_BUILD_ROOT%{_prefix}/bin/* || :
  strip -R .comment $RPM_BUILD_ROOT%{_prefix}/sbin/* || :
  strip -R .comment $RPM_BUILD_ROOT%{_prefix}/libexec/pt_chown || :
  strip -R .comment $RPM_BUILD_ROOT%{_prefix}/%{_lib}/gconv/* || :

  # Hardlink identical locale files together
  ALL_LC="LC_ADDRESS LC_COLLATE LC_CTYPE LC_IDENTIFICATION LC_MEASUREMENT \
	LC_MONETARY LC_NAME LC_NUMERIC LC_PAPER LC_TELEPHONE LC_TIME \
	LC_MESSAGES/SYS_LC_MESSAGES"
  for i in $RPM_BUILD_ROOT%{_prefix}/lib/locale/*; do
    if [ ! -d $i ]; then continue; fi
    for j in $ALL_LC; do
      for k in $RPM_BUILD_ROOT%{_prefix}/lib/locale/*; do
        if [ ! -d $k ]; then continue; fi
        if [ $i = $k ]; then break; fi
        if cmp -s $i/$j $k/$j; then ln -f $k/$j $i/$j; break; fi
      done
    done
  done

  # Build the file list.
  find $RPM_BUILD_ROOT -type f -or -type l |
	sed -e 's|.*/etc|%config &|' \
	    -e 's|.*/gconv/gconv-modules|%verify(not md5 size mtime) %config(noreplace) &|' > rpm.filelist.in
  for n in %{_prefix}/share %{_prefix}/include %{_prefix}/lib/locale; do 
      find ${RPM_BUILD_ROOT}${n} -type d | \
	grep -v '%{_prefix}/share$' | \
	sed "s/^/%dir /" >> rpm.filelist.in
  done

  # primary filelist
  SHARE_LANG='s|.*/share/locale/\([^/_]\+\).*/LC_MESSAGES/.*\.mo|%lang(\1) &|'
  LIB_LANG='s|.*/lib/locale/\([^/_]\+\)|%lang(\1) &|'

  # rpm does not handle %lang() tagged files hardlinked together accross
  # languages very well, temporarily disable
  LIB_LANG=''
  sed -e "s|$RPM_BUILD_ROOT||" -e "$LIB_LANG" -e "$SHARE_LANG" < rpm.filelist.in |
	grep -v '/etc/localtime'  | \
	grep -v '/etc/nsswitch.conf'  | \
	grep -v '/etc/ld.so.conf'  | \
	sort > rpm.filelist

  grep '%{_prefix}/%{_lib}/lib.*_p\.a' < rpm.filelist > profile.filelist || :
  egrep "(%{_prefix}/include)|(%{_infodir})" < rpm.filelist | 
	  grep -v %{_infodir}/dir > devel.filelist

  mv rpm.filelist rpm.filelist.full
  grep -v '%{_prefix}/%{_lib}/lib.*_p.a' rpm.filelist.full | 
	  egrep -v "(%{_prefix}/include)|(%{_infodir})" > rpm.filelist

  grep '%{_prefix}/%{_lib}/lib.*\.a' < rpm.filelist >> devel.filelist
  grep '%{_prefix}/%{_lib}/.*\.o' < rpm.filelist >> devel.filelist
  grep '%{_prefix}/%{_lib}/lib.*\.so' < rpm.filelist >> devel.filelist
  grep '%{_mandir}' < rpm.filelist >> devel.filelist

  mv rpm.filelist rpm.filelist.full
  grep -v '%{_prefix}/%{_lib}/lib.*\.a' < rpm.filelist.full |
	grep -v '%{_prefix}/%{_lib}/.*\.o' |
	grep -v '%{_prefix}/%{_lib}/lib.*\.so'|
	grep -v '%{_mandir}' | 
	grep -v 'nscd' > rpm.filelist
	
  grep '%{_prefix}/bin' < rpm.filelist >> common.filelist
  grep '%{_prefix}/lib/locale' < rpm.filelist >> common.filelist
  grep '%{_prefix}/libexec' < rpm.filelist >> common.filelist
  grep '%{_prefix}/sbin/[^g]' < rpm.filelist >> common.filelist
  grep '%{_prefix}/share' < rpm.filelist >> common.filelist

  mv rpm.filelist rpm.filelist.full
  grep -v '%{_prefix}/bin' < rpm.filelist.full |
	grep -v '%{_prefix}/lib/locale' |
	grep -v '%{_prefix}/libexec' | 
	grep -v '%{_prefix}/sbin/[^g]' |
	grep -v '%{_prefix}/share' > rpm.filelist

  # /etc/localtime - we're proud of our timezone
  rm -f $RPM_BUILD_ROOT/etc/localtime
  cp -f $RPM_BUILD_ROOT%{_prefix}/share/zoneinfo/US/Eastern $RPM_BUILD_ROOT/etc/localtime
  #ln -sf ..%{_prefix}/share/zoneinfo/US/Eastern $RPM_BUILD_ROOT/etc/localtime

  # the last bit: more documentation
  rm -rf documentation
  mkdir documentation
  cp linuxthreads/ChangeLog  documentation/ChangeLog.threads
  cp linuxthreads/Changes documentation/Changes.threads
  cp linuxthreads/README documentation/README.threads
  cp linuxthreads/FAQ.html documentation/FAQ-threads.html
  cp -r linuxthreads/Examples documentation/examples.threads
  cp crypt/README.ufc-crypt documentation/README.ufc-crypt
  cp timezone/README documentation/README.timezone
  cp ChangeLog* documentation
  gzip -9 documentation/ChangeLog*

%postun -p /sbin/ldconfig

%pre devel
  # this used to be a link and it is causing nightmares now
  if [ -L %{_prefix}/include/scsi ] ; then
    rm -f %{_prefix}/include/scsi
  fi

%files -f rpm.filelist
  %doc README NEWS INSTALL FAQ BUGS NOTES PROJECTS CONFORMANCE
  %doc COPYING COPYING.LIB README.template README.libm
  %doc hesiod/README.hesiod

  %defattr(-,root,root)
  %verify(not md5 size mtime) %config(noreplace) /etc/localtime
  %verify(not md5 size mtime) %config(noreplace) /etc/nsswitch.conf
  %verify(not md5 size mtime) %config(noreplace) /etc/ld.so.conf

%ifnarch %{auxarches}
%files -f common.filelist common
  %defattr(-,root,root)
  %doc documentation/*

%files -f devel.filelist devel
  %defattr(-,root,root)

%files -f profile.filelist profile
  %defattr(-,root,root)
%endif

%clean
  rm -rf "$RPM_BUILD_ROOT"
  rm -f *.filelist*

%changelog
* Thu Apr 10 2003 Ryan W. Maple <ryan@guardiandigital.com>
- Release 1.3.0
  - Initial build for EnGarde Secure Community 2 (Feint).

* Thu Mar 20 2003 Ryan W. Maple <ryan@guardiandigital.com>
- Version 2.2.5, Release 1.5.2
  - Bring in XDR library fixes (CERT VU#516825).

* Tue Feb 04 2003 Ryan W. Maple <ryan@guardiandigital.com>
- Version 2.2.5, Release 1.5.1
  - Upgrade to glibc 2.2.5 for EnGarde v1.5 (Thrust).
  - Package taken from old 2.0 development tree.
